@@ -27,12 +27,12 @@
 #define _AVR_IO_H_
 #define __ASSEMBLER__
 #include "avr/iom324.h"
-// instanciate the new core
+// instantiate the new core
 #include "sim_megax4.h"
 
 static avr_t * make()
 {
-	return &SIM_CORENAME.core;
+	return avr_core_allocate(&SIM_CORENAME.core, sizeof(struct mcu_t));
 }
 
 avr_kind_t mega324 = {
